#include <stdlib.h>
#include <curses.h>
#include <signal.h>

#include "poker.h"
#include "common.h"

#define x_pos(card_num) (((card_num)%5) * 8 + 0)
#define y_pos(card_num) (((card_num)/5) ? 12 : 0)

#define WHERE(a,b) ((a) + (5 * ((b) != 0)))
#define MES_Y 1
#define MES_X 1

int	currline;
extern int turn;
void	move_to(int lineno);

/*
 * SHAPES OF THE CARDS
 */
char patterns[]="@&P<";
char	*p_patts[13][7] = {
	{".....", ".....", ".....", "..X..", ".....", ".....", "....." },
	{ "..X..", ".....", ".....", ".....", ".....", ".....", "..X..", },
	{ "..X..", ".....", ".....", "..X..", ".....", ".....", "..X..", },
	{ "X...X", ".....", ".....", ".....", ".....", ".....", "X...X", },
	{ "X...X", ".....", ".....", "..X..", ".....", ".....", "X...X", },
	{ "X...X", ".....", ".....", "X...X", ".....", ".....", "X...X", },
	{ "X...X", ".....", "..X..", "X...X", ".....", ".....", "X...X", },
	{ "X...X", ".....", "..X..", "X...X", "..X..", ".....", "X...X", },
	{ "X...X", "..X..", "X...X", ".....", "X...X", ".....", "X...X" },
	{ "X...X", "..X..", "X...X", ".....", "X...X", "..X..", "X...X" },
	{ ".....", "..XXX", "...X.", "...X.", "X..X.", ".XX..", "....." },
	{ ".....", ".XXX.", "X...X", "X...X", "X.X.X", ".X.X.", "....X" },
	{ ".....", "X..XX", "X.X..", "XX...", "X.X..", "X..XX", "....."}
};

/*
 *  These procedures handle the control of the screen using curses.
 */

WINDOW *_card[10];		/* a seperate window for each card */

/* Do you really want to quit ? */
void
really(void)
{
	int     old_x, old_y;
	char    reply;
	getyx(stdscr, old_y, old_x);
	move(23, 0);
	printw("Do you really want to quit ? ");
	refresh();
	do
		get_input(0, &reply);
	while (reply != 'y' && reply != 'n');
	move(23, 0);
	printw("                             ");
	refresh();
	if (reply == 'y')
		die();
	else {
		move(old_y, old_x);
		refresh();
		redraw_screen();
	}

}

/* What to do if user interupts and really wants to quit Or at end of game. */
void
die(void)
{

	signal(SIGINT, SIG_IGN);
	echo();
	nocrmode();
	mvcur(0, COLS - 1, LINES - 1, 0);
	endwin();
	exit(0);
}


/* init. screen and signals */
void
initialise_screen(void)
{
	initscr();

	/* initscr() has problems when compiled under att SysV  */

	nl();
	noecho();
	crmode();
	/* leaveok(stdscr, TRUE); this line makes cursor location crasy-hunsoo*/
	scrollok(stdscr, FALSE);
	signal(SIGINT, really);
	signal(SIGQUIT, really);
	/* these signals have problems when compiled under att SysV  */

}

void
title_screen(void)
{
	int	i;
	char *starts[]={
	"         +----------------------------------------------------+\n",
	"         |           Welcome to 5 Card Draw Poker             |\n",
	"         |                                                    |\n",
	"         | This is the Final Year Project of Nic Smith.       |\n",
	"         |             csxFROM@uk.ac.kl.seq1                  |\n",
	"         | The implementor of this version can be reached at :|\n",
	"         +----------------------------------------------------+\n"};

	char    key_press;
	clear();
	move(10, 0);

	for (i=0; i < 6; i++) 
		printw(starts[i]);

	printw(IMPLEMENTOR);	/* This is defined in poker.h  */

	printw(starts[i]);

	move(22, 0);
	printw("---More---");
	refresh();
	do
		get_input(0, &key_press);
	while (key_press != ' ')
		/* void */;
}

void
redraw_screen(void)
{
#if 0
#ifdef HPUXs300
	int     count;
	for (count = 0; count < 10; count++)	/* places for the two hands */
		wrefresh(_card[count]);
#else
#endif
#endif
	wrefresh(curscr);
}


/*
 * display instructions and rules. The position of the file used is
 * defined in poker.h  as INSTRUCTIONS
 */

void
instruct(void)
{
	char   inst[100];
	char    reply;


	clear();
	move(1, 0);
	printw("Do you want instructions and rules ? [y/n] : ");
	move(1, 45);
	refresh();
	do
		get_input(0, &reply);
	while (reply != 'y' && reply != 'n');

	printw("%c", reply);
	refresh();
	if (reply == 'y') {
		sprintf(inst, INSTRUCTIONS);
		clear();
		refresh();
		system(inst);
	}
}


/*
 * Set up the screen for the game.
 * i.e. Ten card windows, and positions for the
 * pot and stakes.
 */
void
draw_screen(void)
{
	int     i, j;
	WINDOW	*winp;

	clear();
	refresh();

	/* first, draw the ten windows for the cards  */

	for (i = 0; i < 10; i++) {	/* places for the two hands */
		winp = _card[i] = subwin(stdscr, 11, 9, y_pos(i), x_pos(i));

		scrollok(winp, FALSE);
		leaveok(winp, FALSE);

		/*box(winp, '|', '-'); */
		wmove(winp, 0, 0);
		waddstr(winp, "+-------+");

		for (j=1; j < 10; j++) {
			wmove(winp, j, 0); waddch(winp, '|');
			wmove(winp, j, 8); waddch(winp, '|');
		}
		wmove(winp, j, 0);
		waddstr(winp, "+-------+");

	}

	/* Finally set up other screen messages, such as stake etc. */
	move(11, 5); printw("Yours=");
	move(11, 18); printw("Mine=");
	move(11, 30); printw("Pot=");

	refresh();

	return;
}

/*
 * display the given hand at the top/bottom of sreen 
 * at_bottom, refers to where on the screen, and is boolean 
 */


void
show_hand(playing_card *hand, int at_bottom)
{
	int     i, j, k, tmp;
	char	*cp, patt;
	int	facev, suitv;
	WINDOW	*winp;

	for (i = 0; i < HAND_SIZE; i++) {
		tmp = WHERE(i, at_bottom);
		winp = _card[tmp];

		facev = hand[i].face_value;
		suitv = hand[i].suit_value;

		if (facev == 10) {
			wmove(winp, 1, 1); waddch(winp, '1');
			wmove(winp, 1, 2); waddch(winp, '0');
			wmove(winp, 9, 6); waddch(winp, '1');
			wmove(winp, 9, 7); waddch(winp, '0');
		} else {
			wmove(winp, 1, 1); waddch(winp, hand[i].face);
			wmove(winp, 1, 2); waddch(winp, ' ');
			wmove(winp, 9, 6); waddch(winp, ' ');
			wmove(winp, 9, 7); waddch(winp, hand[i].face);
		}

		patt = patterns[suitv];
		for (j=0; j < 7; j++) {
			wmove(winp, j+2, 1);
			waddstr(winp, "       ");
		}

		for (j=0; j < 7; j++) {
			cp = p_patts[facev-1][j];
			for (k=0; k < 5; k++) {
				if (cp[k] == 'X') {
					wmove(winp, j+2, k+2);
					waddch(winp, patt);
				}
			}
		}

		if (facev > 10) {
			wmove(winp, 2, 2); waddstr(winp, "=====");
		}

		wrefresh(winp);
	};
	refresh();
}

/* wipe out the cards that appeared in the computer's position */
void
clear_my_hand(void)
{
	int     i, j;
	WINDOW	*winp;

	for (i = 0; i < HAND_SIZE; i++) {
		winp = _card[HAND_SIZE+i];
		for (j=1; j<10; j++) {
			wmove(winp, j, 1);
			waddstr(winp, "       ");
		}
		wrefresh(winp);
	}
	refresh();
}

/* Display all the information about the current distribution of wealth */
void 
money(int you, int me, int pot)
{
	move(11, 11); printw("%4d", you);
	move(11, 23); printw("%4d", me);
	move(11, 34); printw("%3d", pot);
	refresh();
}

/* Declare the winner, after a hand is played */
void 
show_winner(int win)
{
	move_to(currline++);
	move_to(currline++);
	if (win == -1)
		printw("It is a draw, the pot stays. ");
	else
		printw(">>>>>  %s win!! <<<<<<", win ? "I" : "You");

	refresh();
}

void
show_value(char *owner, int value)
{
	move_to(currline++);

	printw("---%s card was %s!!", owner, value_to_string(value));
}

/*
 * declare the over all winner, when all of a stake is lost, or no more
 * games are required
 */
void 
show_over_win(int win)
{
	int     i, pause;

	clear_mid();
	move_to(currline++);
	if (win == -1)
		printw("It is a draw.");
	else {
		printw("%sthe over all winner of that game.",
		       win ? "I am " : "You are ");
		move_to(currline++);
		if (win)
			printw("Now isn't that a pity!!");
		else {
			printw("CONGRATULATIONS!");
			move_to(currline++);
			printw("But that was only ONE game.");
		}
	}
	refresh();

	pause = 5 * PAUSE_LEN;
	for (i = 0; i < pause; i++)
		/* void */;
}

/* Computer declares its actions */
void 
i_do(int what)
{
	int     pause;
	move_to(currline++);
	printw("I will ");

	switch (what) {
		case 1: printw("stay."); break;
		case 2: printw("drop my cards."); break;
		case 3: printw("call."); break;
		case 4: case 5: case 6: case 7: case 8:
			printw("bet %2d.", (5 * (what - 3))); break;
		default: printw("raise %2d.", (5 * (what - 8))); break;
	}

	refresh();
	if (what == 1)
		for (pause = 0; pause < PAUSE_LEN; pause++)
		/* void */;
}

/* ask if another game is requred, and get a y/n answer */
int 
ano_yes_no(int source)
{
	char    reply;

	move_to(currline++);
	printw("Do you want another go? ");
	refresh();
	do
		get_input(source, &reply);
	while (reply != 'y' && reply != 'n')
		/* void */ ;

	mvaddch(currline++, FROM, reply);
	refresh();

	return (reply == 'y');
}

/* get and display the user's bet increment	 */
void
show_inc(int raising, int source, int *amount)
{
	move_to(currline++);

	printw("How much do you want to %s (1-5):",
	       raising ? "raise" : "bet");
	refresh();
	get_increment(source, amount);
	move_to(currline++);
	printw("%2d", *amount);
	refresh();

}

/* ask the user if he wants to stay, drop or bet, and get the reply */
int 
choose_SDB(int source, int *amount)
{
	char    choice;
	int     response;

	move_to(currline++);
	printw("Do you wish to Stay,Drop or Bet:");
	refresh();

	get_SDB(source, &choice);
	move_to(currline);
	printw("%c", choice);
	refresh();

	if (choice == 'b')
		show_inc(0, source, amount);

	switch (choice) {
		case 's': /* Staying */
			response = 1;
			break;
		case 'd': /* dropping */
			response = 2;
			break;
		case 'b': /* betting */
			response = (*amount / 5) + 3;	/* bet this */

	}
	return response;
}

/* ask user if he wants to raise, drop or call */
int 
choose_RCD(int source, int *amount, int turn)
{
	char    choice;
	int     response;

	move_to(currline++);
	printw("Do you wish to %sDrop or Call:",
		(turn == 6) ? "" : "Raise,");
	refresh();

	do
		get_RCD(source, &choice, turn); /* turn inserted by hunsoo */
	while (turn == 6 && choice == 'r');	/* disallow raise on turn 6 */

	move_to(currline++);
	printw("%c", choice);
	refresh();
	if (choice == 'r')
		show_inc(1, source, amount);

	switch (choice) {
		case 'c':		/* Staying */
			response = 3;
			break;
		case 'd':		/* dropping */
			response = 2;
			break;
		case 'r':		/* betting */
			response = (*amount / 5) + 8;

	}
	return response;
}

static char clean_line[801];

/* Clear the message lines at the middle of the screen  */
void
clear_mid(void)
{
	static int firstcalled = 0;
	int	i;

	if (firstcalled == 0) {
		for (i=0; i<40; i++) {
			clean_line[i] = ' ';
		}
		clean_line[38] = 0;
		firstcalled++;
	}
	currline = 0;

	for (i=0; i<24; i++) {
		move(i, FROM);
		printw(clean_line);
	}
	refresh();
}

void 
inform_ante(int pot) /* inform user that ante is required */
{
	int     pause;

	clear_mid();
	move_to(currline++);
	printw("We ante%s. ", (pot > 10) ? " again" : "");
	refresh();
	for (pause = 0; pause < PAUSE_LEN; pause++)
		/* void */ ;
	/* clear_mid(); *//* commented by hunsoo */
}

/*
 * Following functions are for getting users
 * rejected cards.
 */
void
ask_which(void)
{
	move_to(currline++);
	printw("Which cards do you want to reject(1-5)");
	refresh();
}

/* delete the user's input  */
void
delete_line(void)
{
	move_to(currline);
	printw(clean_line);
	move_to(currline);
	refresh();
}

/* Does computer choose to stay, drop, bet, raise or call ? */
void
show_choice(char choice, int count)
{
	int     across;
	across = FROM + (count * 2);

	mvaddch(currline, across, choice);
	refresh();

}

/* Computer declares how many cards it takes */
void
i_take(int num)
{
	int     pause;

	move_to(currline++);
	printw("I will take %d card%s",
		num, (num != 1) ? "s." : ".");

	refresh();
	for (pause = 0; pause < PAUSE_LEN; pause++)
		/* void */;
	if (0) {
		move_to(--currline);
		printw("                          ");
	}
	refresh();

}

void
move_to(int lineno)
{
	move(lineno, FROM);
	printw(clean_line);
	move(lineno, FROM);
	if (lineno >= 22) {
		move(0, FROM);
		printw(clean_line);
		move(0, FROM);
		currline = 0;
	}
}

void
show_turn(void)
{
	move(11, 0); printw("[%d]", turn);
	return;
}
